#ifndef _controlParameter_h
#define _controlParameter_h

#include "pluginterfaces/base/ustring.h"

namespace Steinberg {
    namespace Vst {
        
        class ControlParameter : public Parameter {
        public:
            ControlParameter(const TChar* pName, uchar id, UnitID unitID = kRootUnitId, ParamValue defValue = 0., int32 stepCount = 0, int32 flags = ParameterInfo::kCanAutomate, const TChar* pUnits = 0) {
                info.flags = flags;
                info.id = (int32) id;
                Steinberg::UString(info.title, USTRINGSIZE (info.title)).assign(USTRING(pName));
                info.unitId = unitID;
                info.defaultNormalizedValue = defValue;
                info.stepCount = stepCount;
                Steinberg::UString(info.units, USTRINGSIZE (info.units)).assign(USTRING(pUnits));                     
            }
        };
        
        class ControlDiscreteParameter : public StringListParameter {
        public:
            ControlDiscreteParameter(const TChar* pName, uchar id, UnitID unitID = kRootUnitId, ParamValue defValue = 0.5, int32 flags = ParameterInfo::kCanAutomate, const TChar* pUnits = 0) 
            : StringListParameter(pName, id, pUnits, flags, unitID) {
                info.defaultNormalizedValue = defValue;
            }
        };
        
        class ControlFreqParameter : public RangeParameter {
        public:
            ControlFreqParameter(const TChar* pName, uchar id, UnitID unitID = kRootUnitId, ParamValue defPlainValue = 44000., ParamValue minPlainValue = 100., ParamValue maxPlainValue = 1000000., int32 stepCount = 999900, 
                          int32 flags = ParameterInfo::kCanAutomate, const TChar* pUnits = STR16("Hz")) : RangeParameter(pName, id, pUnits, minPlainValue, maxPlainValue, defPlainValue, stepCount, flags, unitID) {
                
            }
            
            virtual void toString (ParamValue valueNormalized, String128 string) const {
                    Parameter::toString(toPlain(valueNormalized) / 100, string);
            }
            
            virtual bool fromString (const TChar* string, ParamValue &valueNormalized) const {
                String wrapper((TChar*)string);
                
                if (wrapper.scanFloat(valueNormalized)) {
                    valueNormalized *= 100;
                    if (valueNormalized < getMin())
                        valueNormalized = getMin();
                    else if (valueNormalized > getMax())
                        valueNormalized = getMax();
                    valueNormalized = toNormalized(valueNormalized);
                    return true;
                }
                return false;
            }
        };

    }
}


#endif
