#include "dx7ctrlr.h"

namespace Steinberg {
    namespace Vst {
        tresult PLUGIN_API DX7Controller::initialize (FUnknown* context) {
            tresult result = EditControllerEx1::initialize (context);
            if (result != kResultOk) {
                return result;
            }
            
            /* Unit Creation */
            createUnit(1, STR16("OP1"));
            createUnit(2, STR16("OP2"));
            createUnit(3, STR16("OP3"));
            createUnit(4, STR16("OP4"));
            createUnit(5, STR16("OP5"));
            createUnit(6, STR16("OP6"));
            createUnit(7, STR16("Global"));
            
            /* Parameter Creation */
            ControlParameter* param;
            ControlDiscreteParameter* discreteParam;
            ControlFreqParameter* freqParam;
            uchar pID = 0;
            /* Global Parameters */
            param = new ControlParameter(STR16("Volume"), pID, 7, 0.15);
            parameters.addParameter(param);
            
            param = new ControlParameter(STR16("Pan"), ++pID, 7, 0.5);
            parameters.addParameter(param);
            
            discreteParam = new ControlDiscreteParameter(STR16("Algorithm"), ++pID, 7, 0.);
            String algorithmStrings[32] = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32"};
            for (int i = 0; i < 32; ++i)
                discreteParam->appendString(algorithmStrings[i]);
            parameters.addParameter(discreteParam);
            
            discreteParam = new ControlDiscreteParameter(STR16("Feedback"), ++pID, 7, 0.);
            String feedbackStrings[8] = {"0", "1", "2", "3", "4", "5", "6", "7"};
            for (int i = 0; i < 8; ++i)
                discreteParam->appendString(feedbackStrings[i]);
            parameters.addParameter(discreteParam);
            
            discreteParam = new ControlDiscreteParameter(STR16("LFO Wave"), ++pID, 7, 0.);
            discreteParam->setUnitID(7);
            String lfowStrings[6] = {"Triangle", "Saw Down", "Saw up", "Square", "Sine", "S&H"};
            for (int i = 0; i < 6; ++i)
                discreteParam->appendString(lfowStrings[i]);
            parameters.addParameter(discreteParam);
            
            param = new ControlParameter(STR16("LFO Speed"), ++pID, 7, 0.);
            parameters.addParameter(param);
            
            param = new ControlParameter(STR16("LFO Delay"), ++pID, 7, 0.);
            parameters.addParameter(param);
            
            discreteParam = new ControlDiscreteParameter(STR16("LFO Sync"), ++pID, 7, 0.);
            String toggleStrings[2] = {"Off", "On"};
            for (int i = 0; i < 2; ++i)
                discreteParam->appendString(toggleStrings[i]);
            parameters.addParameter(discreteParam);
            
            param = new ControlParameter(STR16("LFO Pitch"), ++pID, 7, 0.);
            parameters.addParameter(param);
            
            discreteParam = new ControlDiscreteParameter(STR16("OSC Sync"), ++pID, 7, 0.);
            for (int i = 0; i < 2; ++i)
                discreteParam->appendString(toggleStrings[i]);
            parameters.addParameter(discreteParam);
            
            param = new ControlParameter(STR16("PEG Rate 1"), ++pID, 7, 1.);
            parameters.addParameter(param);
            
            param = new ControlParameter(STR16("PEG Lvl 1"), ++pID, 7, 0.5);
            parameters.addParameter(param);
            
            param = new ControlParameter(STR16("PEG Rate 2"), ++pID, 7, 1.);
            parameters.addParameter(param);
            
            param = new ControlParameter(STR16("PEG Lvl 2"), ++pID, 7, 0.5);
            parameters.addParameter(param);
            
            param = new ControlParameter(STR16("PEG Rate 3"), ++pID, 7, 1.);
            parameters.addParameter(param);
            
            param = new ControlParameter(STR16("PEG Lvl 3"), ++pID, 7, 0.5);
            parameters.addParameter(param);
            
            param = new ControlParameter(STR16("PEG Rate 4"), ++pID, 7, 1.);
            parameters.addParameter(param);
            
            param = new ControlParameter(STR16("PEG Lvl 4"), ++pID, 7, 0.5);
            parameters.addParameter(param);
            
            discreteParam = new ControlDiscreteParameter(STR16("Transpose"), ++pID, 7, 0.5);
            String transStrings[49] = { "-24", "-23", "-22", "-21", "-20", "-19", "-18", "-17", "-16", "-15", "-14", "-13", "-12", "-11", "-10", "-9", "-8", "-7", "-6", "-5", "-4", "-3", "-2", "-1", 
                                        "0", 
                                        "+1", "+2", "+3", "+4", "+5", "+6", "+7", "+8", "+9", "+10", "+11", "+12", "+13", "+14", "+15", "+16", "+17", "+18", "+19", "+20", "+21", "+22", "+23", "+24"};
            for (int i = 0; i < 49; ++i)
                discreteParam->appendString(transStrings[i]);
            parameters.addParameter(discreteParam);
            
            /* Operator Specific Parameters */
            for (int i = 1; i < 7; ++i) {
                param = new ControlParameter(STR16("Volume"), ++pID, i, i == 1 ? 1.0 : 0.);
                parameters.addParameter(param);
                
                discreteParam = new ControlDiscreteParameter(STR16("Active"), ++pID, i, 1.);
                for (int j = 0; j < 2; ++j)
                    discreteParam->appendString(toggleStrings[j]);
                parameters.addParameter(discreteParam);
                
                param = new ControlParameter(STR16("LFO AMS"), ++pID, i, 0.);
                parameters.addParameter(param);
                
                discreteParam = new ControlDiscreteParameter(STR16("OSC Mode"), ++pID, i, 0.);
                String modeStrings[2] = {"Ratio", "Fixed"};
                for (int j = 0; j < 2; ++j)
                    discreteParam->appendString(modeStrings[j]);
                parameters.addParameter(discreteParam);
                
                discreteParam = new ControlDiscreteParameter(STR16("Ratio Coarse"), ++pID, i, 0.045);
                String coarseStrings[32] = {"0.5", "1", "2", "3", "4", "5", "6", "7", 
                                            "8", "9", "10", "11", "12", "13", "14", "15", 
                                            "16", "17", "18", "19", "20", "21", "22", "23", 
                                            "24", "25", "26", "27", "28", "29", "30", "31"};
                for (int j = 0; j < 32; ++j)
                    discreteParam->appendString(coarseStrings[j]);
                parameters.addParameter(discreteParam);
                
                param = new ControlParameter(STR16("Ratio Fine"), ++pID, i, 0.);
                parameters.addParameter(param);
                
                freqParam = new ControlFreqParameter(STR16("Fixed Freq"), ++pID, i, 44000.);
                parameters.addParameter(freqParam);
                                
                param = new ControlParameter(STR16("EG Rate 1"), ++pID, i, 1.);
                parameters.addParameter(param);
                
                param = new ControlParameter(STR16("EG Lvl 1"), ++pID, i, 1.);
                parameters.addParameter(param);
                
                param = new ControlParameter(STR16("EG Rate 2"), ++pID, i, 1.);
                parameters.addParameter(param);
                
                param = new ControlParameter(STR16("EG Lvl 2"), ++pID, i, 1.);
                parameters.addParameter(param);
                
                
                param = new ControlParameter(STR16("EG Rate 3"), ++pID, i, 1.);
                parameters.addParameter(param);
                
                param = new ControlParameter(STR16("EG Lvl 3"), ++pID, i, 1.);
                parameters.addParameter(param);
                
                
                param = new ControlParameter(STR16("EG Rate 4"), ++pID, i, 1.);
                parameters.addParameter(param);
                
                param = new ControlParameter(STR16("EG Lvl 4"), ++pID, i, 0.);
                parameters.addParameter(param);
                
                discreteParam = new ControlDiscreteParameter(STR16("Kbd Rate Amt"), ++pID, i, 0.);
                String krsStrings[8] = {"0", "1", "2", "3", "4", "5", "6", "7"};
                for (int j = 0; j < 8; ++j)
                    discreteParam->appendString(krsStrings[j]);
                parameters.addParameter(discreteParam);
                
                discreteParam = new ControlDiscreteParameter(STR16("Kbd Vel Sens"), ++pID, i, 0.);
                String kvsStrings[8] = {"0", "1", "2", "3", "4", "5", "6", "7"};
                for (int j = 0; j < 8; ++j)
                    discreteParam->appendString(kvsStrings[j]);
                parameters.addParameter(discreteParam);
            }
            
            paramCount = pID + 1;
            return result;
        }
        
        tresult PLUGIN_API DX7Controller::setComponentState(IBStream* state) {
            if (state) {
                float pTemp = 0.f;
                for (int i = 0; i < paramCount; ++i) {
                    if (state->read (&pTemp, sizeof(float)) != kResultOk) {
                        return kResultFalse;
                    }
                    #if BYTEORDER == kBigEndian
                    SWAP_32 (pTemp)
                    #endif
                    setParamNormalized (i, pTemp);
                }
            }
            return kResultOk;
        }
        
        tresult PLUGIN_API DX7Controller::setState(IBStream* state) {            
            return kResultTrue;
        }
        
        tresult PLUGIN_API DX7Controller::getState(IBStream* state) {
            return kResultTrue;        
        }
        
        void DX7Controller::createUnit(int32 id, const TChar* uName) {
            UnitInfo unitInfo;
            Unit* unit;
            
            unitInfo.id = id;
            unitInfo.parentUnitId = kRootUnitId;
            Steinberg::UString(unitInfo.name, USTRINGSIZE(unitInfo.name)).assign(USTRING(uName));
            unitInfo.programListId = kNoProgramListId;
            unit = new Unit(unitInfo);
            addUnit(unit);
        }

    }
}
