#include "dx7proc.h"

namespace Steinberg {
    namespace Vst {
        
        DX7Processor::DX7Processor(): currentProcessMode(-1) {
            setControllerClass (DX7ControllerUID);
            connection = new DX7Connection();
            
            /* Setup Data Model */
            uchar pID = 0;
            parameters.insert(std::pair<int32, ProcessParameter*>(pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_VOL, 0.15)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_PAN, 0.5)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(GLOBAL_PARAM, GLB_ALG, 0., 7)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(GLOBAL_PARAM, GLB_FBK, 0., 7)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(GLOBAL_PARAM, GLB_LFO_WAV, 0., 5)));            
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_LFO_SPD, 0.)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_LFO_DEL, 0.)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(GLOBAL_PARAM, GLB_LFO_SYNC, 0., 1)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_LFO_PITCH, 0.)));            
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(GLOBAL_PARAM, GLB_OSC_SYNC, 0., 1)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_PEG_R1, 1.)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_PEG_L1, 0.5)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_PEG_R2, 1.)));            
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_PEG_L2, 0.5)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_PEG_R3, 1.)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_PEG_L3, 0.5)));
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_PEG_R4, 1.)));            
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(GLOBAL_PARAM, GLB_PEG_L4, 0.5)));            
            parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(GLOBAL_PARAM, GLB_TRANS, 0.5, 48)));
            
            /* Operator Specific Parameters */
            for (int i = 1; i < 7; ++i) {
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_VOL, i == 1 ? 1.0 : 0.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(i, OP_ACT, 1., 1)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_LFO_AMS, 0.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(i, OP_OSC_MODE, 0., 1)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessCoarseParameter(i, OP_OSC_RATIO_COARSE	, 0.045)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_OSC_RATIO_FINE, 0.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessFreqParameter(i, OP_OSC_FIXED_FREQ, 0.04390439043904390439043904390439)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_EG_R1, 1.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_EG_L1, 1.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_EG_R2, 1.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_EG_L2, 1.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_EG_R3, 1.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_EG_L3, 1.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_EG_R4, 1.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessContinuousParameter(i, OP_EG_L4, 0.)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(i, KBD_RS, 0., 7)));
                parameters.insert(std::pair<int32, ProcessParameter*>(++pID, new ProcessDiscreteParameter(i, KBD_VS, 0., 7)));
            } 
            
            paramCount = pID + 1;
            
            /* Send initial state */
            if (connection->isOnline())
                for (int i = 0; i < paramCount; ++i)
                    connection->writeParameter(getParameter(i));
        }
        
        DX7Processor::~DX7Processor() {
            /* Delete Parameters */
            ParamMap::const_iterator end = parameters.end(); 
            for (ParamMap::const_iterator it = parameters.begin(); it != end; ++it) {
                delete it->second;
            }
            
            delete connection; 
        }
        
        tresult PLUGIN_API DX7Processor::initialize (FUnknown* context) {            
            tresult result = AudioEffect::initialize (context);
            
            if (result != kResultOk)
                return result;
            
            addAudioOutput(STR16("Stereo Out"), SpeakerArr::kStereo);
            addEventInput(STR16("Event In"), 1);
            
            return kResultOk;
        }
        
        tresult PLUGIN_API DX7Processor::process(ProcessData& data) {
            if (connection->isOnline()) {
                /* Handle Incoming Parameter Changes */
                IParameterChanges* paramChanges = data.inputParameterChanges;
                if (paramChanges) {
                    int32 numParamsChanged = paramChanges->getParameterCount();
                    for (int32 i = 0; i < numParamsChanged; ++i) {
                        IParamValueQueue* paramQueue = paramChanges->getParameterData(i);
                        if (paramQueue) {
                            int32 offsetSamples;
                            double value;
                            int32 numPoints = paramQueue->getPointCount ();
                            uint8 paramID = paramQueue->getParameterId();
                            if (paramQueue->getPoint(numPoints - 1,  offsetSamples, value) == kResultTrue) {
                                updateParameter(paramID, (float)value);
                                connection->writeParameter(getParameter(paramID));                      
                            }
                        }
                    }
                }
                
                /* Handle Incoming (Midi) Events */
                IEventList* eventList = data.inputEvents;
                if (eventList) {
                    int32 numEvent = eventList->getEventCount();
                    for (int32 i = 0; i < numEvent; i++) {
                        Event event;
                        if (eventList->getEvent(i, event) == kResultOk) {
                            connection->writeEvent(event);
                        }
                    }
                }
            }
            
            return kResultOk;
        }
        
        tresult PLUGIN_API DX7Processor::setState(IBStream* state) {
            float pTemp = 0.f;
            for (int i = 0; i < paramCount; ++i) {
                if (state->read (&pTemp, sizeof(float)) != kResultOk) {
                    return kResultFalse;
                }
                #if BYTEORDER == kBigEndian
                    SWAP_32(pTemp)
                    #endif
                updateParameter(i, pTemp);
            }
            
            return kResultOk;
        }
        
        tresult PLUGIN_API DX7Processor::getState(IBStream* state) {            
            float pTemp;
            for (int i = 0; i < paramCount; ++i) {
                pTemp = getParameterValue(i);  
            
                #if BYTEORDER == kBigEndian
                SWAP_32 (pTemp)
                #endif
                state->write(&pTemp, sizeof (float));
            }
            
            return kResultOk;
        }
        
        tresult PLUGIN_API DX7Processor::setupProcessing(ProcessSetup& newSetup) {
            
            currentProcessMode = newSetup.processMode;
            
            return AudioEffect::setupProcessing(newSetup);
        }
        
        void DX7Processor::updateParameter(uint8 id, float value) {
            parameters.find(id)->second->update(value);
        }
        
        ProcessParameter* DX7Processor::getParameter(uint8 id) {
            return parameters.find(id)->second;
        }
        
        float DX7Processor::getParameterValue(uint8 id) {
            return parameters.find(id)->second->getValue();
        }
    }
}
