#include "dx7uart.h"

namespace Steinberg {
    namespace Vst {
        
        DX7Connection::DX7Connection() {   
            setup();
        }
        
        DX7Connection::~DX7Connection() {
            quit();
        }
        
        void DX7Connection::setup() {
            char* pSerials[MAX_DEVICES + 1];
            char serials[MAX_DEVICES][64];
            int	numDevs;
            pSerials[MAX_DEVICES] = NULL;
            
            for(int i = 0; i < MAX_DEVICES; ++i) {
                pSerials[i] = serials[i];
            }
            
            ftStatus = FT_ListDevices(pSerials, &numDevs, FT_LIST_ALL | FT_OPEN_BY_SERIAL_NUMBER);
            
            /* Assumes the first connected FTDI device is our dx7fpga */
            if(ftStatus == FT_OK)
                for(int i = 0; i < MAX_DEVICES; ++i) {
                    ftStatus = FT_OpenEx(serials[i], FT_OPEN_BY_SERIAL_NUMBER, &ftHandle);
                    if (ftStatus == FT_OK) {
                        ftStatus = FT_ResetDevice(ftHandle);
                        ftStatus = FT_Purge(ftHandle, FT_PURGE_RX | FT_PURGE_TX);
                        UCHAR LatencyTimer = 2;
                        ftStatus = FT_SetLatencyTimer(ftHandle, LatencyTimer);
                        DWORD transferSize = 64;
                        ftStatus = FT_SetUSBParameters(ftHandle, transferSize, transferSize);
                        ftStatus = FT_SetBaudRate(ftHandle, 1562500);
                        ftStatus = FT_SetDataCharacteristics(ftHandle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE);
                        ftStatus = FT_SetFlowControl(ftHandle, FT_FLOW_NONE, 0x1, 0x0);
                        break;
                    }
                }
        }
        
        void DX7Connection::quit() {
            if(ftHandle != NULL) {
                FT_Close(ftHandle);
                ftHandle = NULL;
            }
        }
        
        void DX7Connection::writeParameter(ProcessParameter* param) {
            DWORD 	dwBytesWritten;
            ftStatus = FT_Write(ftHandle, param->getByteRep(), BUF_SIZE, &dwBytesWritten);
        }
        
        void DX7Connection::writeEvent(Event event) {
            uint16 type = event.type;
            if (type == Event::kNoteOnEvent | type == Event::kNoteOffEvent) {
                DWORD 	dwBytesWritten;
                uchar byteRep[BUF_SIZE];
                uchar velocity;
                byteRep[0] = NOTE_EVENT;
                byteRep[4] = 0;
                byteRep[5] = 0;
                
                switch(type) {
                    case Event::kNoteOnEvent:
                        byteRep[1] = 0x90;
                        byteRep[2] = (uchar) event.noteOn.pitch;
                        velocity = event.noteOn.velocity * 127;
                        velocity = velocity < 127? velocity : 127; 
                        byteRep[3] = velocity;
                        break;
                    case Event::kNoteOffEvent:
                        byteRep[1] = 0x80;
                        byteRep[2] = (uchar) event.noteOff.pitch;
                        velocity = event.noteOff.velocity * 127;
                        velocity = velocity < 127? velocity : 127; 
                        byteRep[3] = velocity;
                        break;
                }
                ftStatus = FT_Write(ftHandle, byteRep, BUF_SIZE, &dwBytesWritten);
            }
        }
        
        bool DX7Connection::isOnline() {
            return ftStatus == FT_OK;
        }
    }
}